2005-12-31  Vincent Untz  <vuntz {at} gnome.org>

	Make code more solid and fix icons not visible. Code was updated from
	the xfce fork of this code.
	Fix bug #108864 and probably some other bugs...

	* libegg/tray/eggtraymanager.c: use gi18n.h
	(egg_tray_manager_init): init more variables
	(egg_tray_manager_finalize): free more stuff
	(egg_tray_manager_handle_dock_request): don't handle notifications we
	already got, do a size request for the socket so that it has the
	correct size
	(egg_tray_manager_check_running_xscreen): fix test
1 /* eggtraymanager.c
 
2  * Copyright (C) 2002 Anders Carlsson <andersca@gnu.org>
 
3  *
 
4  * This library is free software; you can redistribute it and/or
 
5  * modify it under the terms of the GNU Lesser General Public
 
6  * License as published by the Free Software Foundation; either
 
7  * version 2 of the License, or (at your option) any later version.
 
8  *
 
9  * This library is distributed in the hope that it will be useful,
 
10  * but WITHOUT ANY WARRANTY; without even the implied warranty of
 
11  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 
12  * Lesser General Public License for more details.
 
13  *
 
14  * You should have received a copy of the GNU Lesser General Public
 
15  * License along with this library; if not, write to the
 
16  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 
17  * Boston, MA 02111-1307, USA.
 
18  */
 
19  
20 #include <config.h>
 
21 #include <string.h>
 
22 #include <libintl.h>
 
23  
24 #include "eggtraymanager.h"
 
25  
26 #include <gdkconfig.h>
 
27 #include <glib/gi18n.h>
 
28 #if defined (GDK_WINDOWING_X11)
 
29 #include <gdk/gdkx.h>
 
30 #include <X11/Xatom.h>  
31 #elif defined (GDK_WINDOWING_WIN32)
 
32 #include <gdk/gdkwin32.h>
 
33 #endif
 
34 #include <gtk/gtkinvisible.h>
 
35 #include <gtk/gtksocket.h>
 
36 #include <gtk/gtkwindow.h>
 
37  
38 #include "eggmarshalers.h"
 
39  
40 /* Signals */
 
41 enum
 
42 {
 
43   TRAY_ICON_ADDED,
 
44   TRAY_ICON_REMOVED,
 
45   MESSAGE_SENT,
 
46   MESSAGE_CANCELLED,
 
47   LOST_SELECTION,
 
48   LAST_SIGNAL
 
49 };
 
50  
51 enum {
 
52   PROP_0,
 
53   PROP_ORIENTATION
 
54 };
 
55  
56 typedef struct
 
57 {
 
58   long id, len;
 
59   long remaining_len;
 
60   
 
61   long timeout;
 
62   char *str;
 
63 #ifdef GDK_WINDOWING_X11
 
64   Window window;
 
65 #endif
 
66 } PendingMessage;
 
67  
68 static GObjectClass *parent_class = NULL;
 
69 static guint manager_signals[LAST_SIGNAL] = { 0 };
 
70  
71 #define SYSTEM_TRAY_REQUEST_DOCK    0
 
72 #define SYSTEM_TRAY_BEGIN_MESSAGE   1
 
73 #define SYSTEM_TRAY_CANCEL_MESSAGE  2
 
74  
75 #define SYSTEM_TRAY_ORIENTATION_HORZ 0
 
76 #define SYSTEM_TRAY_ORIENTATION_VERT 1
 
77  
78 #ifdef GDK_WINDOWING_X11
 
79 static gboolean egg_tray_manager_check_running_xscreen (Screen *xscreen);
 
80 #endif
 
81  
82 static void egg_tray_manager_init (EggTrayManager *manager);
 
83 static void egg_tray_manager_class_init (EggTrayManagerClass *klass);
 
84  
85 static void egg_tray_manager_finalize     (GObject      *object);
 
86 static void egg_tray_manager_set_property (GObject      *object,
 
87 					   guint         prop_id,
 
88 					   const GValue *value,
 
89 					   GParamSpec   *pspec);
 
90 static void egg_tray_manager_get_property (GObject      *object,
 
91 					   guint         prop_id,
 
92 					   GValue       *value,
 
93 					   GParamSpec   *pspec);
 
94  
95 static void egg_tray_manager_unmanage (EggTrayManager *manager);
 
96  
97 GType
 
98 egg_tray_manager_get_type (void)
 
99 {
 
100   static GType our_type = 0;
 
101  
102   if (our_type == 0)
 
103     {
 
104       static const GTypeInfo our_info =
 
105       {
 
106 	sizeof (EggTrayManagerClass),
 
107 	(GBaseInitFunc) NULL,
 
108 	(GBaseFinalizeFunc) NULL,
 
109 	(GClassInitFunc) egg_tray_manager_class_init,  
110 	NULL, /* class_finalize */
 
111 	NULL, /* class_data */
 
112 	sizeof (EggTrayManager),
 
113 	0,    /* n_preallocs */
 
114 	(GInstanceInitFunc) egg_tray_manager_init
 
115       };
 
116  
117       our_type = g_type_register_static (G_TYPE_OBJECT, "EggTrayManager", &our_info, 0);
 
118     }
 
119  
120   return our_type;
 
121  
122 }
 
123  
124 static void
 
125 egg_tray_manager_init (EggTrayManager *manager)
 
126 {
 
127   manager->invisible = NULL;
 
128   manager->socket_table = g_hash_table_new (NULL, NULL);
 
129 }
 
130  
131 static void
 
132 egg_tray_manager_class_init (EggTrayManagerClass *klass)
 
133 {
 
134   GObjectClass *gobject_class;
 
135   
 
136   parent_class = g_type_class_peek_parent (klass);  
137   gobject_class = (GObjectClass *)klass;
 
138  
139   gobject_class->finalize = egg_tray_manager_finalize;
 
140   gobject_class->set_property = egg_tray_manager_set_property;
 
141   gobject_class->get_property = egg_tray_manager_get_property;  
142  
143   g_object_class_install_property (gobject_class,
 
144 				   PROP_ORIENTATION,
 
145 				   g_param_spec_enum ("orientation",
 
146 						      _("Orientation"),
 
147 						      _("The orientation of the tray."),
 
148 						      GTK_TYPE_ORIENTATION,
 
149 						      GTK_ORIENTATION_HORIZONTAL,
 
150 						      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
151   
 
152   manager_signals[TRAY_ICON_ADDED] =
 
153     g_signal_new ("tray_icon_added",
 
154 		  G_OBJECT_CLASS_TYPE (klass),
 
155 		  G_SIGNAL_RUN_LAST,
 
156 		  G_STRUCT_OFFSET (EggTrayManagerClass, tray_icon_added),
 
157 		  NULL, NULL,
 
158 		  g_cclosure_marshal_VOID__OBJECT,
 
159 		  G_TYPE_NONE, 1,
 
160 		  GTK_TYPE_SOCKET);
 
161  
162   manager_signals[TRAY_ICON_REMOVED] =
 
163     g_signal_new ("tray_icon_removed",
 
164 		  G_OBJECT_CLASS_TYPE (klass),
 
165 		  G_SIGNAL_RUN_LAST,
 
166 		  G_STRUCT_OFFSET (EggTrayManagerClass, tray_icon_removed),
 
167 		  NULL, NULL,
 
168 		  g_cclosure_marshal_VOID__OBJECT,
 
169 		  G_TYPE_NONE, 1,
 
170 		  GTK_TYPE_SOCKET);
 
171   manager_signals[MESSAGE_SENT] =
 
172     g_signal_new ("message_sent",
 
173 		  G_OBJECT_CLASS_TYPE (klass),
 
174 		  G_SIGNAL_RUN_LAST,
 
175 		  G_STRUCT_OFFSET (EggTrayManagerClass, message_sent),
 
176 		  NULL, NULL,
 
177 		  _egg_marshal_VOID__OBJECT_STRING_LONG_LONG,
 
178 		  G_TYPE_NONE, 4,
 
179 		  GTK_TYPE_SOCKET,
 
180 		  G_TYPE_STRING,
 
181 		  G_TYPE_LONG,
 
182 		  G_TYPE_LONG);  
183   manager_signals[MESSAGE_CANCELLED] =
 
184     g_signal_new ("message_cancelled",
 
185 		  G_OBJECT_CLASS_TYPE (klass),
 
186 		  G_SIGNAL_RUN_LAST,
 
187 		  G_STRUCT_OFFSET (EggTrayManagerClass, message_cancelled),
 
188 		  NULL, NULL,
 
189 		  _egg_marshal_VOID__OBJECT_LONG,
 
190 		  G_TYPE_NONE, 2,
 
191 		  GTK_TYPE_SOCKET,
 
192 		  G_TYPE_LONG);
 
193   manager_signals[LOST_SELECTION] =
 
194     g_signal_new ("lost_selection",
 
195 		  G_OBJECT_CLASS_TYPE (klass),
 
196 		  G_SIGNAL_RUN_LAST,
 
197 		  G_STRUCT_OFFSET (EggTrayManagerClass, lost_selection),  
198 		  NULL, NULL,
 
199 		  g_cclosure_marshal_VOID__VOID,
 
200 		  G_TYPE_NONE, 0);
 
201  
202 #if defined (GDK_WINDOWING_X11)
 
203   /* Nothing */
 
204 #elif defined (GDK_WINDOWING_WIN32)  
205   g_warning ("Port eggtraymanager to Win32");
 
206 #else
 
207   g_warning ("Port eggtraymanager to this GTK+ backend");
 
208 #endif
 
209 }
 
210  
211 static void
 
212 egg_tray_manager_finalize (GObject *object)
 
213 {
 
214   EggTrayManager *manager;
 
215   
 
216   manager = EGG_TRAY_MANAGER (object);
 
217  
218   egg_tray_manager_unmanage (manager);
 
219  
220   g_list_free (manager->messages);
 
221   g_hash_table_destroy (manager->socket_table);
 
222   
 
223   G_OBJECT_CLASS (parent_class)->finalize (object);
 
224 }
 
225  
226 static void
 
227 egg_tray_manager_set_property (GObject      *object,
 
228 			       guint         prop_id,
 
229 			       const GValue *value,
 
230 			       GParamSpec   *pspec)
 
231 {
 
232   EggTrayManager *manager = EGG_TRAY_MANAGER (object);
 
233  
234   switch (prop_id)
 
235     {
 
236     case PROP_ORIENTATION:
 
237       egg_tray_manager_set_orientation (manager, g_value_get_enum (value));
 
238       break;
 
239     default:
 
240       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 
241       break;
 
242     }
 
243 }
 
244  
245 static void
 
246 egg_tray_manager_get_property (GObject    *object,
 
247 			       guint       prop_id,
 
248 			       GValue     *value,
 
249 			       GParamSpec *pspec)
 
250 {
 
251   EggTrayManager *manager = EGG_TRAY_MANAGER (object);
 
252  
253   switch (prop_id)
 
254     {
 
255     case PROP_ORIENTATION:
 
256       g_value_set_enum (value, manager->orientation);
 
257       break;
 
258     default:
 
259       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 
260       break;
 
261     }
 
262 }
 
263  
264 EggTrayManager *
 
265 egg_tray_manager_new (void)
 
266 {
 
267   EggTrayManager *manager;
 
268  
269   manager = g_object_new (EGG_TYPE_TRAY_MANAGER, NULL);  
270  
271   return manager;
 
272 }
 
273  
274 #ifdef GDK_WINDOWING_X11
 
275  
276 static gboolean
 
277 egg_tray_manager_plug_removed (GtkSocket       *socket,
 
278 			       EggTrayManager  *manager)
 
279 {
 
280   Window *window;
 
281  
282   window = g_object_get_data (G_OBJECT (socket), "egg-tray-child-window");
 
283  
284   g_hash_table_remove (manager->socket_table, GINT_TO_POINTER (*window));
 
285   g_object_set_data (G_OBJECT (socket), "egg-tray-child-window",
 
286 		     NULL);
 
287   
 
288   g_signal_emit (manager, manager_signals[TRAY_ICON_REMOVED], 0, socket);
 
289  
290   /* This destroys the socket. */
 
291   return FALSE;
 
292 }
 
293  
294 static void
 
295 egg_tray_manager_handle_dock_request (EggTrayManager       *manager,
 
296 				      XClientMessageEvent  *xevent)
 
297 {
 
298   GtkWidget *socket;
 
299   Window *window;
 
300   GtkRequisition req;
 
301  
302   if (g_hash_table_lookup (manager->socket_table, GINT_TO_POINTER (xevent->data.l[2])))
 
303     {
 
304       /* We already got this notification earlier, ignore this one */
 
305       return;
 
306     }
 
307   
 
308   socket = gtk_socket_new ();
 
309   
 
310   /* We need to set the child window here
 
311    * so that the client can call _get functions
 
312    * in the signal handler
 
313    */
 
314   window = g_new (Window, 1);
 
315   *window = xevent->data.l[2];
 
316       
 
317   g_object_set_data_full (G_OBJECT (socket),
 
318 			  "egg-tray-child-window",
 
319 			  window, g_free);
 
320   g_signal_emit (manager, manager_signals[TRAY_ICON_ADDED], 0,
 
321 		 socket);
 
322  
323   /* Add the socket only if it's been attached */
 
324   if (GTK_IS_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (socket))))
 
325     {
 
326       g_signal_connect (socket, "plug_removed",
 
327 			G_CALLBACK (egg_tray_manager_plug_removed), manager);
 
328       
 
329       gtk_socket_add_id (GTK_SOCKET (socket), *window);
 
330  
331       g_hash_table_insert (manager->socket_table, GINT_TO_POINTER (*window), socket);
 
332  
333       /*
 
334        * Make sure the icons have a meaningfull size ...
 
335        */ 
 
336       req.width = req.height = 1;
 
337       gtk_widget_size_request (socket, &req);
 
338       /*
 
339       if ((req.width < 16) || (req.height < 16))
 
340       {
 
341           gint nw = MAX (24, req.width);
 
342           gint nh = MAX (24, req.height);
 
343           g_warning (_("tray icon has requested a size of (%i x %i), resizing to (%i x %i)"), 
 
344                       req.width, req.height, nw, nh);
 
345           gtk_widget_set_size_request(icon, nw,  nh);
 
346       }
 
347       */
 
348       gtk_widget_show(socket);
 
349     }
 
350   else
 
351     gtk_widget_destroy (socket);
 
352 }
 
353  
354 static void
 
355 pending_message_free (PendingMessage *message)
 
356 {
 
357   g_free (message->str);
 
358   g_free (message);
 
359 }
 
360  
361 static void
 
362 egg_tray_manager_handle_message_data (EggTrayManager       *manager,
 
363 				       XClientMessageEvent  *xevent)
 
364 {
 
365   GList *p;
 
366   int len;
 
367   
 
368   /* Try to see if we can find the
 
369    * pending message in the list
 
370    */
 
371   for (p = manager->messages; p; p = p->next)
 
372     {
 
373       PendingMessage *msg = p->data;
 
374  
375       if (xevent->window == msg->window)
 
376 	{
 
377 	  /* Append the message */
 
378 	  len = MIN (msg->remaining_len, 20);
 
379  
380 	  memcpy ((msg->str + msg->len - msg->remaining_len),
 
381 		  &xevent->data, len);
 
382 	  msg->remaining_len -= len;
 
383  
384 	  if (msg->remaining_len == 0)
 
385 	    {
 
386 	      GtkSocket *socket;
 
387  
388 	      socket = g_hash_table_lookup (manager->socket_table, GINT_TO_POINTER (msg->window));
 
389  
390 	      if (socket)
 
391 		{
 
392 		  g_signal_emit (manager, manager_signals[MESSAGE_SENT], 0,
 
393 				 socket, msg->str, msg->id, msg->timeout);
 
394 		}
 
395 	      manager->messages = g_list_remove_link (manager->messages,
 
396 						      p);
 
397 	      
 
398 	      pending_message_free (msg);
 
399 	    }
 
400  
401 	  return;
 
402 	}
 
403     }
 
404 }
 
405  
406 static void
 
407 egg_tray_manager_handle_begin_message (EggTrayManager       *manager,  
408 				       XClientMessageEvent  *xevent)
 
409 {
 
410   GList *p;
 
411   PendingMessage *msg;
 
412  
413   /* Check if the same message is
 
414    * already in the queue and remove it if so
 
415    */
 
416   for (p = manager->messages; p; p = p->next)
 
417     {
 
418       PendingMessage *msg = p->data;
 
419  
420       if (xevent->window == msg->window &&
 
421 	  xevent->data.l[4] == msg->id)  
422 	{
 
423 	  /* Hmm, we found it, now remove it */
 
424 	  pending_message_free (msg);
 
425 	  manager->messages = g_list_remove_link (manager->messages, p);
 
426 	  break;
 
427 	}
 
428     }
 
429  
430   /* Now add the new message to the queue */
 
431   msg = g_new0 (PendingMessage, 1);
 
432   msg->window = xevent->window;
 
433   msg->timeout = xevent->data.l[2];
 
434   msg->len = xevent->data.l[3];
 
435   msg->id = xevent->data.l[4];
 
436   msg->remaining_len = msg->len;
 
437   msg->str = g_malloc (msg->len + 1);
 
438   msg->str[msg->len] = '\0';
 
439   manager->messages = g_list_prepend (manager->messages, msg);
 
440 }
 
441  
442 static void
 
443 egg_tray_manager_handle_cancel_message (EggTrayManager       *manager,
 
444 					XClientMessageEvent  *xevent)
 
445 {
 
446   GtkSocket *socket;
 
447   
 
448   socket = g_hash_table_lookup (manager->socket_table, GINT_TO_POINTER (xevent->window));
 
449   
 
450   if (socket)
 
451     {
 
452       g_signal_emit (manager, manager_signals[MESSAGE_CANCELLED], 0,
 
453 		     socket, xevent->data.l[2]);
 
454     }
 
455 }
 
456  
457 static GdkFilterReturn
 
458 egg_tray_manager_handle_event (EggTrayManager       *manager,
 
459 			       XClientMessageEvent  *xevent)
 
460 {
 
461   switch (xevent->data.l[1])
 
462     {
 
463     case SYSTEM_TRAY_REQUEST_DOCK:
 
464       egg_tray_manager_handle_dock_request (manager, xevent);
 
465       return GDK_FILTER_REMOVE;
 
466  
467     case SYSTEM_TRAY_BEGIN_MESSAGE:
 
468       egg_tray_manager_handle_begin_message (manager, xevent);
 
469       return GDK_FILTER_REMOVE;
 
470  
471     case SYSTEM_TRAY_CANCEL_MESSAGE:
 
472       egg_tray_manager_handle_cancel_message (manager, xevent);
 
473       return GDK_FILTER_REMOVE;
 
474     default:
 
475       break;
 
476     }
 
477  
478   return GDK_FILTER_CONTINUE;
 
479 }
 
480  
481 static GdkFilterReturn
 
482 egg_tray_manager_window_filter (GdkXEvent *xev, GdkEvent *event, gpointer data)
 
483 {
 
484   XEvent *xevent = (GdkXEvent *)xev;
 
485   EggTrayManager *manager = data;
 
486  
487   if (xevent->type == ClientMessage)
 
488     {
 
489       if (xevent->xclient.message_type == manager->opcode_atom)
 
490 	{
 
491 	  return egg_tray_manager_handle_event (manager, (XClientMessageEvent *)xevent);
 
492 	}
 
493       else if (xevent->xclient.message_type == manager->message_data_atom)
 
494 	{
 
495 	  egg_tray_manager_handle_message_data (manager, (XClientMessageEvent *)xevent);
 
496 	  return GDK_FILTER_REMOVE;
 
497 	}
 
498     }
 
499   else if (xevent->type == SelectionClear)
 
500     {
 
501       g_signal_emit (manager, manager_signals[LOST_SELECTION], 0);
 
502       egg_tray_manager_unmanage (manager);
 
503     }
 
504   return GDK_FILTER_CONTINUE;
 
505 }
 
506  
507 #endif  
 
508  
509 static void
 
510 egg_tray_manager_unmanage (EggTrayManager *manager)
 
511 {
 
512 #ifdef GDK_WINDOWING_X11
 
513   Display *display;
 
514   guint32 timestamp;
 
515   GtkWidget *invisible;
 
516  
517   if (manager->invisible == NULL)
 
518     return;
 
519  
520   invisible = manager->invisible;
 
521   g_assert (GTK_IS_INVISIBLE (invisible));
 
522   g_assert (GTK_WIDGET_REALIZED (invisible));
 
523   g_assert (GDK_IS_WINDOW (invisible->window));
 
524   
 
525   display = GDK_WINDOW_XDISPLAY (invisible->window);
 
526   
 
527   if (XGetSelectionOwner (display, manager->selection_atom) ==
 
528       GDK_WINDOW_XWINDOW (invisible->window))
 
529     {
 
530       timestamp = gdk_x11_get_server_time (invisible->window);      
 
531       XSetSelectionOwner (display, manager->selection_atom, None, timestamp);
 
532     }
 
533  
534   gdk_window_remove_filter (invisible->window, egg_tray_manager_window_filter, manager);  
 
535  
536   manager->invisible = NULL; /* prior to destroy for reentrancy paranoia */
 
537   gtk_widget_destroy (invisible);
 
538   g_object_unref (G_OBJECT (invisible));
 
539 #endif
 
540 }
 
541  
542 static void
 
543 egg_tray_manager_set_orientation_property (EggTrayManager *manager)
 
544 {
 
545 #ifdef GDK_WINDOWING_X11
 
546   gulong data[1];
 
547  
548   if (!manager->invisible || !manager->invisible->window)
 
549     return;
 
550  
551   g_assert (manager->orientation_atom != None);
 
552  
553   data[0] = manager->orientation == GTK_ORIENTATION_HORIZONTAL ?
 
554 		SYSTEM_TRAY_ORIENTATION_HORZ :
 
555 		SYSTEM_TRAY_ORIENTATION_VERT;
 
556  
557   XChangeProperty (GDK_WINDOW_XDISPLAY (manager->invisible->window),
 
558 		   GDK_WINDOW_XWINDOW (manager->invisible->window),
 
559 		   manager->orientation_atom,
 
560 		   XA_CARDINAL, 32,
 
561 		   PropModeReplace,
 
562 		   (guchar *) &data, 1);
 
563 #endif
 
564 }
 
565  
566 #ifdef GDK_WINDOWING_X11
 
567  
568 static gboolean
 
569 egg_tray_manager_manage_xscreen (EggTrayManager *manager, Screen *xscreen)
 
570 {
 
571   GtkWidget *invisible;
 
572   char *selection_atom_name;
 
573   guint32 timestamp;  
574   GdkScreen *screen;
 
575   
 
576   g_return_val_if_fail (EGG_IS_TRAY_MANAGER (manager), FALSE);
 
577   g_return_val_if_fail (manager->screen == NULL, FALSE);
 
578  
579   /* If there's already a manager running on the screen
 
580    * we can't create another one.
 
581    */
 
582 #if 0
 
583   if (egg_tray_manager_check_running_xscreen (xscreen))
 
584     return FALSE;
 
585 #endif
 
586   screen = gdk_display_get_screen (gdk_x11_lookup_xdisplay (DisplayOfScreen (xscreen)),
 
587 				   XScreenNumberOfScreen (xscreen));
 
588   
 
589   invisible = gtk_invisible_new_for_screen (screen);
 
590   gtk_widget_realize (invisible);
 
591   
 
592   gtk_widget_add_events (invisible, GDK_PROPERTY_CHANGE_MASK | GDK_STRUCTURE_MASK);
 
593  
594   selection_atom_name = g_strdup_printf ("_NET_SYSTEM_TRAY_S%d",
 
595 					 XScreenNumberOfScreen (xscreen));
 
596   manager->selection_atom = XInternAtom (DisplayOfScreen (xscreen), selection_atom_name, False);
 
597  
598   g_free (selection_atom_name);
 
599  
600   manager->orientation_atom = XInternAtom (DisplayOfScreen (xscreen),
 
601 					   "_NET_SYSTEM_TRAY_ORIENTATION",
 
602 					   FALSE);
 
603   egg_tray_manager_set_orientation_property (manager);
 
604   
 
605   timestamp = gdk_x11_get_server_time (invisible->window);
 
606   XSetSelectionOwner (DisplayOfScreen (xscreen), manager->selection_atom,
 
607 		      GDK_WINDOW_XWINDOW (invisible->window), timestamp);
 
608  
609   /* Check if we were could set the selection owner successfully */
 
610   if (XGetSelectionOwner (DisplayOfScreen (xscreen), manager->selection_atom) ==
 
611       GDK_WINDOW_XWINDOW (invisible->window))
 
612     {
 
613       XClientMessageEvent xev;
 
614  
615       xev.type = ClientMessage;
 
616       xev.window = RootWindowOfScreen (xscreen);
 
617       xev.message_type = XInternAtom (DisplayOfScreen (xscreen), "MANAGER", False);
 
618  
619       xev.format = 32;
 
620       xev.data.l[0] = timestamp;
 
621       xev.data.l[1] = manager->selection_atom;
 
622       xev.data.l[2] = GDK_WINDOW_XWINDOW (invisible->window);
 
623       xev.data.l[3] = 0;	/* manager specific data */
 
624       xev.data.l[4] = 0;	/* manager specific data */
 
625  
626       XSendEvent (DisplayOfScreen (xscreen),
 
627 		  RootWindowOfScreen (xscreen),
 
628 		  False, StructureNotifyMask, (XEvent *)&xev);
 
629  
630       manager->invisible = invisible;
 
631       g_object_ref (G_OBJECT (manager->invisible));
 
632       
 
633       manager->opcode_atom = XInternAtom (DisplayOfScreen (xscreen),
 
634 					  "_NET_SYSTEM_TRAY_OPCODE",
 
635 					  False);
 
636  
637       manager->message_data_atom = XInternAtom (DisplayOfScreen (xscreen),
 
638 						"_NET_SYSTEM_TRAY_MESSAGE_DATA",
 
639 						False);
 
640  
641       /* Add a window filter */
 
642       gdk_window_add_filter (invisible->window, egg_tray_manager_window_filter, manager);
 
643       return TRUE;
 
644     }
 
645   else
 
646     {
 
647       gtk_widget_destroy (invisible);
 
648  
 
649       return FALSE;
 
650     }
 
651 }
 
652  
653 #endif
 
654  
655 gboolean
 
656 egg_tray_manager_manage_screen (EggTrayManager *manager,
 
657 				GdkScreen      *screen)
 
658 {
 
659   g_return_val_if_fail (GDK_IS_SCREEN (screen), FALSE);
 
660   g_return_val_if_fail (manager->screen == NULL, FALSE);
 
661  
662 #ifdef GDK_WINDOWING_X11
 
663   return egg_tray_manager_manage_xscreen (manager, 
 
664 					  GDK_SCREEN_XSCREEN (screen));
 
665 #else
 
666   return FALSE;
 
667 #endif
 
668 }
 
669  
670 #ifdef GDK_WINDOWING_X11
 
671  
672 static gboolean
 
673 egg_tray_manager_check_running_xscreen (Screen *xscreen)
 
674 {
 
675   Atom selection_atom;
 
676   char *selection_atom_name;
 
677  
678   selection_atom_name = g_strdup_printf ("_NET_SYSTEM_TRAY_S%d",
 
679 					 XScreenNumberOfScreen (xscreen));
 
680   selection_atom = XInternAtom (DisplayOfScreen (xscreen), selection_atom_name, False);
 
681   g_free (selection_atom_name);
 
682  
683   if (XGetSelectionOwner (DisplayOfScreen (xscreen), selection_atom) != None)
 
684     return TRUE;
 
685   else
 
686     return FALSE;
 
687 }  
688  
689 #endif
 
690  
691 gboolean
 
692 egg_tray_manager_check_running (GdkScreen *screen)
 
693 {
 
694   g_return_val_if_fail (GDK_IS_SCREEN (screen), FALSE);
 
695  
696 #ifdef GDK_WINDOWING_X11
 
697   return egg_tray_manager_check_running_xscreen (GDK_SCREEN_XSCREEN (screen));
 
698 #else
 
699   return FALSE;
 
700 #endif
 
701 }
 
702  
703 char *
 
704 egg_tray_manager_get_child_title (EggTrayManager *manager,
 
705 				  EggTrayManagerChild *child)
 
706 {
 
707   char *retval = NULL;
 
708 #ifdef GDK_WINDOWING_X11
 
709   Window *child_window;
 
710   Atom utf8_string, atom, type;
 
711   int result;
 
712   int format;
 
713   gulong nitems;
 
714   gulong bytes_after;
 
715   guchar *val;
 
716  
717   g_return_val_if_fail (EGG_IS_TRAY_MANAGER (manager), NULL);
 
718   g_return_val_if_fail (GTK_IS_SOCKET (child), NULL);
 
719   
 
720   child_window = g_object_get_data (G_OBJECT (child),
 
721 				    "egg-tray-child-window");
 
722  
723   utf8_string = XInternAtom (GDK_DISPLAY (), "UTF8_STRING", False);
 
724   atom = XInternAtom (GDK_DISPLAY (), "_NET_WM_NAME", False);
 
725  
726   gdk_error_trap_push ();
 
727  
728   result = XGetWindowProperty (GDK_DISPLAY (),
 
729 			       *child_window,  
730 			       atom,
 
731 			       0, G_MAXLONG,
 
732 			       False, utf8_string,
 
733 			       &type, &format, &nitems,
 
734 			       &bytes_after, (guchar **)&val);
 
735   
 
736   if (gdk_error_trap_pop () || result != Success)
 
737     return NULL;
 
738  
739   if (type != utf8_string ||
 
740       format != 8 ||
 
741       nitems == 0)
 
742     {
 
743       if (val)
 
744 	XFree (val);
 
745       return NULL;
 
746     }
 
747  
748   if (!g_utf8_validate (val, nitems, NULL))
 
749     {
 
750       XFree (val);
 
751       return NULL;
 
752     }
 
753  
754   retval = g_strndup (val, nitems);
 
755  
756   XFree (val);
 
757 #endif
 
758   return retval;
 
759 }
 
760  
761 void
 
762 egg_tray_manager_set_orientation (EggTrayManager *manager,
 
763 				  GtkOrientation  orientation)
 
764 {
 
765   g_return_if_fail (EGG_IS_TRAY_MANAGER (manager));
 
766  
767   if (manager->orientation != orientation)
 
768     {
 
769       manager->orientation = orientation;
 
770  
771       egg_tray_manager_set_orientation_property (manager);
 
772  
773       g_object_notify (G_OBJECT (manager), "orientation");
 
774     }
 
775 }
 
776  
777 GtkOrientation
 
778 egg_tray_manager_get_orientation (EggTrayManager *manager)
 
779 {
 
780   g_return_val_if_fail (EGG_IS_TRAY_MANAGER (manager), GTK_ORIENTATION_HORIZONTAL);
 
781  
782   return manager->orientation;
 
783 }