2008-12-11  Philip Withnall  <philip {at} tecnocode.co.uk>

	* libegg/smclient/eggdesktopfile.c 
	(set_startup_notification_timeout):
	* libegg/print-operation/modules/printbackends/cups/eggprintbackendcups.c
	(cups_get_printer_list):
	* libegg/recent-files/egg-recent-model.c
	(egg_recent_model_monitor):
	* libegg/recentchooser/eggrecentmanager.c
	(egg_recent_manager_init):
	* libegg/tray/teststatusicon.c (main): Use g_timeout_add_seconds
	where possible to allow for event grouping. Bug #564156.

1 #include "eggstatusicon.h"
 
2 #include <gtk/gtk.h>
 
3 #include <libgnomeui/gnome-icon-theme.h>
 
4  
5 typedef enum
  
6 {
 
7   TEST_STATUS_FILE,
 
8   TEST_STATUS_DIRECTORY
 
9 } TestStatus;
 
10  
11 static GnomeIconTheme *icon_theme = NULL;
 
12 static TestStatus status = TEST_STATUS_FILE;
  
13  
14 static void
 
15 update_icon (EggStatusIcon *status_icon)
 
16 {
 
17   gchar *icon_name;
 
18   gchar *path;
 
19   gchar *tooltip;
 
20  
21   if (status == TEST_STATUS_FILE)
 
22     {
 
23       icon_name = "gnome-fs-regular";
 
24       tooltip = "Regular File";
 
25     }
 
26   else
 
27     {
 
28       icon_name = "gnome-fs-directory";
 
29       tooltip = "Directory";
 
30     }
 
31  
32   egg_status_icon_set_tooltip (status_icon, tooltip, NULL);
 
33  
34   path = gnome_icon_theme_lookup_icon (icon_theme, icon_name,
 
35 				       egg_status_icon_get_size (status_icon),
 
36 				       NULL, NULL);
 
37   if (path)
 
38     {
 
39       GdkPixbuf *pixbuf;
 
40       GError *error = NULL;
 
41  
42       pixbuf = gdk_pixbuf_new_from_file (path, &error);
 
43       if (error)
 
44         {
 
45 	  g_warning ("Failed to load '%s': %s", path, error->message);
 
46 	  g_error_free (error);
 
47         }
 
48       else
 
49 	{
 
50 	  egg_status_icon_set_from_pixbuf (status_icon, pixbuf);
 
51 	  g_object_unref (pixbuf);
 
52 	}
 
53  
54       g_free (path);
 
55     }
 
56   else
 
57     {
 
58       g_warning ("Failed to lookup icon '%s'", icon_name);
 
59     }
 
60 }
 
61  
62 static gboolean  
63 timeout_handler (gpointer data)
 
64 {
 
65   EggStatusIcon *icon = EGG_STATUS_ICON (data);
 
66  
67   if (status == TEST_STATUS_FILE)
 
68     status = TEST_STATUS_DIRECTORY;
 
69   else
 
70     status = TEST_STATUS_FILE;
 
71  
72   update_icon (icon);  
73  
74   return TRUE;
 
75 }
 
76  
77 static void
 
78 blink_toggle_toggled (GtkToggleButton *toggle,
 
79 		      EggStatusIcon   *icon)
 
80 {
 
81   egg_status_icon_set_is_blinking (icon, gtk_toggle_button_get_active (toggle));
 
82 }
 
83  
84 static void
 
85 icon_activated (EggStatusIcon *icon)
 
86 {
 
87   GtkWidget *dialog;
 
88   GtkWidget *toggle;
 
89  
90   dialog = g_object_get_data (G_OBJECT (icon), "test-status-icon-dialog");  
91   if (dialog == NULL)
 
92     {
 
93       dialog = gtk_message_dialog_new (NULL, 0,
 
94 				       GTK_MESSAGE_QUESTION,
 
95 				       GTK_BUTTONS_CLOSE,
 
96 				       "You wanna test the status icon ?");
 
97  
98       g_object_set_data_full (G_OBJECT (icon), "test-status-icon-dialog",
 
99 			      dialog, (GDestroyNotify) gtk_widget_destroy);
 
100  
101       g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_hide), NULL);
 
102       g_signal_connect (dialog, "delete_event", G_CALLBACK (gtk_widget_hide_on_delete), NULL);
 
103  
104       toggle = gtk_toggle_button_new_with_mnemonic ("_Blink the icon");
 
105       gtk_box_pack_end (GTK_BOX (GTK_DIALOG (dialog)->vbox), toggle, TRUE, TRUE, 6);
 
106       gtk_widget_show (toggle);
 
107  
108       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), egg_status_icon_get_is_blinking (icon));
 
109       g_signal_connect (toggle, "toggled", G_CALLBACK (blink_toggle_toggled), icon);
 
110     }
 
111  
112   gtk_window_present (GTK_WINDOW (dialog));
 
113 }
  
114  
115 int
 
116 main (int argc, char **argv)
 
117 {
 
118   EggStatusIcon *icon;
 
119  
120   gtk_init (&argc, &argv);
 
121  
122   icon_theme = gnome_icon_theme_new ();
 
123  
124   icon = egg_status_icon_new ();
 
125   update_icon (icon);
 
126  
127   egg_status_icon_set_is_blinking (EGG_STATUS_ICON (icon), TRUE);
 
128  
129   g_signal_connect (icon, "activate",
 
130 		    G_CALLBACK (icon_activated), NULL);
 
131  
132   g_timeout_add_seconds (2, timeout_handler, icon);
 
133  
134   gtk_main ();
 
135  
136   g_object_unref (icon_theme);
 
137  
138   return 0;
 
139 }
 
